@@ -1,22 +1,21 @@
+from flake8 import utils as stdin_utils
+from pathlib import Path
+
 import ast
 import builtins
 import inspect
 import sys
 
-from flake8 import utils as stdin_utils
-
-if sys.version_info >= (3, 8):
-    NamedExpr = ast.NamedExpr
-else:  # There was no walrus operator before python3.8
-    NamedExpr = type('NamedExpr', (ast.AST,), {})
-
 
 class BuiltinsChecker:
     name = 'flake8_builtins'
     version = '1.5.2'
-    assign_msg = 'A001 variable "{0}" is shadowing a python builtin'
-    argument_msg = 'A002 argument "{0}" is shadowing a python builtin'
-    class_attribute_msg = 'A003 class attribute "{0}" is shadowing a python builtin'
+    assign_msg = 'A001 variable "{0}" is shadowing a Python builtin'
+    argument_msg = 'A002 argument "{0}" is shadowing a Python builtin'
+    class_attribute_msg = 'A003 class attribute "{0}" is shadowing a Python builtin'
+    import_msg = 'A004 import statement "{0}" is shadowing a Python builtin'
+    module_name_msg = 'A005 the module is shadowing a Python builtin module "{0}"'
+    lambda_argument_msg = 'A006 lambda argument "{0}" is shadowing a Python builtin'
 
     names = []
     ignore_list = {
@@ -25,6 +24,7 @@ class BuiltinsChecker:
         'credits',
         '_',
     }
+    ignored_module_names = set()
 
     def __init__(self, tree, filename):
         self.tree = tree
@@ -39,6 +39,13 @@ class BuiltinsChecker:
             comma_separated_list=True,
             help='A comma separated list of builtins to skip checking',
         )
+        option_manager.add_option(
+            '--builtins-allowed-modules',
+            metavar='builtins',
+            parse_from_config=True,
+            comma_separated_list=True,
+            help='A comma separated list of builtin module names to allow',
+        )
 
     @classmethod
     def parse_options(cls, options):
@@ -52,12 +59,26 @@ class BuiltinsChecker:
         if flake8_builtins:
             cls.names.update(flake8_builtins)
 
+        if options.builtins_allowed_modules is not None:
+            cls.ignored_module_names.update(options.builtins_allowed_modules)
+
+        if hasattr(sys, 'stdlib_module_names'):
+            # stdlib_module_names is only available in Python 3.10+
+            known_module_names = sys.stdlib_module_names
+            cls.module_names = {
+                m for m in known_module_names if m not in cls.ignored_module_names
+            }
+        else:
+            cls.module_names = set()
+
     def run(self):
         tree = self.tree
 
         if self.filename == 'stdin':
             lines = stdin_utils.stdin_get_value()
             tree = ast.parse(lines)
+        else:
+            yield from self.check_module_name(self.filename)
 
         for statement in ast.walk(tree):
             for child in ast.iter_child_nodes(statement):
@@ -87,12 +108,15 @@ class BuiltinsChecker:
 
         value = None
         for statement in ast.walk(tree):
-            if isinstance(statement, (ast.Assign, ast.AnnAssign, NamedExpr)):
+            if isinstance(statement, (ast.Assign, ast.AnnAssign, ast.NamedExpr)):
                 value = self.check_assignment(statement)
 
             elif isinstance(statement, function_nodes):
                 value = self.check_function_definition(statement)
 
+            elif isinstance(statement, ast.Lambda):
+                value = self.check_lambda_definition(statement)
+
             elif isinstance(statement, for_nodes):
                 value = self.check_for_loop(statement)
 
@@ -161,6 +185,20 @@ class BuiltinsChecker:
                     variable=arg.arg,
                 )
 
+    def check_lambda_definition(self, statement):
+        all_arguments = []
+        all_arguments.extend(statement.args.args)
+        all_arguments.extend(getattr(statement.args, 'kwonlyargs', []))
+        all_arguments.extend(getattr(statement.args, 'posonlyargs', []))
+
+        for arg in all_arguments:
+            if isinstance(arg, ast.arg) and arg.arg in self.names:
+                yield self.error(
+                    arg,
+                    message=self.lambda_argument_msg,
+                    variable=arg.arg,
+                )
+
     def check_for_loop(self, statement):
         stack = [statement.target]
         while stack:
@@ -223,20 +261,45 @@ class BuiltinsChecker:
 
     def check_import(self, statement):
         for name in statement.names:
-            if name.asname in self.names:
-                yield self.error(statement, variable=name.asname)
+            collision = None
+            if name.name in self.names and name.asname is None:
+                collision = name.name
+            elif name.asname in self.names:
+                collision = name.asname
+            if collision:
+                yield self.error(
+                    statement,
+                    message=self.import_msg,
+                    variable=collision,
+                )
 
     def check_class(self, statement):
         if statement.name in self.names:
             yield self.error(statement, variable=statement.name)
 
-    def error(self, statement, variable, message=None):
+    def error(self, statement=None, variable=None, message=None):
         if not message:
             message = self.assign_msg
 
+        # lineno and col_offset must be integers
         return (
-            statement.lineno,
-            statement.col_offset,
+            statement.lineno if statement else 0,
+            statement.col_offset if statement else 0,
             message.format(variable),
             type(self),
         )
+
+    def check_module_name(self, filename: str):
+        if not self.module_names:
+            return
+        path = Path(filename)
+        if path.name == '__init__.py':
+            module_name = path.parent.name
+        else:
+            module_name = path.name.removesuffix('.py')
+        if module_name in self.module_names:
+            yield self.error(
+                None,
+                module_name,
+                message=self.module_name_msg,
+            )
